@@ -1,23 +1,13 @@
-"""
-    sphinx.cmd.quickstart
-    ~~~~~~~~~~~~~~~~~~~~~
-
-    Quickly setup documentation source to work with Sphinx.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Quickly setup documentation source to work with Sphinx."""
 
 import argparse
 import locale
 import os
-import re
 import sys
 import time
-import warnings
 from collections import OrderedDict
 from os import path
-from typing import Any, Callable, Dict, List, Pattern, Union
+from typing import Any, Callable, Dict, List, Union
 
 # try to import readline, unix specific enhancement
 try:
@@ -29,22 +19,18 @@ try:
         readline.parse_and_bind("tab: complete")
         USE_LIBEDIT = False
 except ImportError:
+    readline = None
     USE_LIBEDIT = False
 
 from docutils.utils import column_width
 
 import sphinx.locale
 from sphinx import __display_version__, package_dir
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import __
-from sphinx.util.console import (  # type: ignore
-    colorize, bold, red, turquoise, nocolor, color_terminal
-)
+from sphinx.util.console import bold, color_terminal, colorize, nocolor, red  # type: ignore
 from sphinx.util.osutil import ensuredir
 from sphinx.util.template import SphinxRenderer
 
-TERM_ENCODING = getattr(sys.stdin, 'encoding', None)  # RemovedInSphinx40Warning
-
 EXTENSIONS = OrderedDict([
     ('autodoc', __('automatically insert docstrings from modules')),
     ('doctest', __('automatically test code snippets in doctest blocks')),
@@ -54,10 +40,8 @@ EXTENSIONS = OrderedDict([
     ('imgmath', __('include math, rendered as PNG or SVG images')),
     ('mathjax', __('include math, rendered in the browser by MathJax')),
     ('ifconfig', __('conditional inclusion of content based on config values')),
-    ('viewcode',
-     __('include links to the source code of documented Python objects')),
-    ('githubpages',
-     __('create .nojekyll file to publish the document on GitHub pages')),
+    ('viewcode', __('include links to the source code of documented Python objects')),
+    ('githubpages', __('create .nojekyll file to publish the document on GitHub pages')),
 ])
 
 DEFAULTS = {
@@ -103,6 +87,12 @@ def is_path(x: str) -> str:
     return x
 
 
+def is_path_or_empty(x: str) -> str:
+    if x == '':
+        return x
+    return is_path(x)
+
+
 def allow_empty(x: str) -> str:
     return x
 
@@ -129,8 +119,7 @@ def boolean(x: str) -> bool:
 
 def suffix(x: str) -> str:
     if not (x[0:1] == '.' and len(x) > 1):
-        raise ValidationError(__("Please enter a file suffix, "
-                                 "e.g. '.rst' or '.txt'."))
+        raise ValidationError(__("Please enter a file suffix, e.g. '.rst' or '.txt'."))
     return x
 
 
@@ -138,30 +127,6 @@ def ok(x: str) -> str:
     return x
 
 
-def term_decode(text: Union[bytes, str]) -> str:
-    warnings.warn('term_decode() is deprecated.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-
-    if isinstance(text, str):
-        return text
-
-    # Use the known encoding, if possible
-    if TERM_ENCODING:
-        return text.decode(TERM_ENCODING)
-
-    # If ascii is safe, use it with no warning
-    if text.decode('ascii', 'replace').encode('ascii', 'replace') == text:
-        return text.decode('ascii')
-
-    print(turquoise(__('* Note: non-ASCII characters entered '
-                       'and terminal encoding unknown -- assuming '
-                       'UTF-8 or Latin-1.')))
-    try:
-        return text.decode()
-    except UnicodeDecodeError:
-        return text.decode('latin1')
-
-
 def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] = nonempty) -> Union[str, bool]:  # NOQA
     while True:
         if default is not None:
@@ -173,8 +138,11 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
             # sequence (see #5335).  To avoid the problem, all prompts are not colored
             # on libedit.
             pass
-        else:
+        elif readline:
+            # pass input_mode=True if readline available
             prompt = colorize(COLOR_QUESTION, prompt, input_mode=True)
+        else:
+            prompt = colorize(COLOR_QUESTION, prompt, input_mode=False)
         x = term_input(prompt).strip()
         if default and not x:
             x = default
@@ -187,22 +155,27 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
     return x
 
 
-def convert_python_source(source: str, rex: Pattern = re.compile(r"[uU]('.*?')")) -> str:
-    # remove Unicode literal prefixes
-    warnings.warn('convert_python_source() is deprecated.',
-                  RemovedInSphinx40Warning)
-    return rex.sub('\\1', source)
-
-
 class QuickstartRenderer(SphinxRenderer):
     def __init__(self, templatedir: str) -> None:
         self.templatedir = templatedir or ''
         super().__init__()
 
+    def _has_custom_template(self, template_name: str) -> bool:
+        """Check if custom template file exists.
+
+        Note: Please don't use this function from extensions.
+              It will be removed in the future without deprecation period.
+        """
+        template = path.join(self.templatedir, path.basename(template_name))
+        if self.templatedir and path.exists(template):
+            return True
+        else:
+            return False
+
     def render(self, template_name: str, context: Dict) -> str:
-        user_template = path.join(self.templatedir, path.basename(template_name))
-        if self.templatedir and path.exists(user_template):
-            return self.render_from_file(user_template, context)
+        if self._has_custom_template(template_name):
+            custom_template = path.join(self.templatedir, path.basename(template_name))
+            return self.render_from_file(custom_template, context)
         else:
             return super().render(template_name, context)
 
@@ -228,16 +201,16 @@ def ask_user(d: Dict) -> None:
     """
 
     print(bold(__('Welcome to the Sphinx %s quickstart utility.')) % __display_version__)
-    print(__('''
-Please enter values for the following settings (just press Enter to
-accept a default value, if one is given in brackets).'''))
+    print()
+    print(__('Please enter values for the following settings (just press Enter to\n'
+             'accept a default value, if one is given in brackets).'))
 
     if 'path' in d:
-        print(bold(__('''
-Selected root path: %s''') % d['path']))
+        print()
+        print(bold(__('Selected root path: %s')) % d['path'])
     else:
-        print(__('''
-Enter the root path for documentation.'''))
+        print()
+        print(__('Enter the root path for documentation.'))
         d['path'] = do_prompt(__('Root path for the documentation'), '.', is_path)
 
     while path.isfile(path.join(d['path'], 'conf.py')) or \
@@ -247,70 +220,68 @@ Enter the root path for documentation.'''))
                       'selected root path.')))
         print(__('sphinx-quickstart will not overwrite existing Sphinx projects.'))
         print()
-        d['path'] = do_prompt(__('Please enter a new root path (or just Enter '
-                                 'to exit)'), '', is_path)
+        d['path'] = do_prompt(__('Please enter a new root path (or just Enter to exit)'),
+                              '', is_path_or_empty)
         if not d['path']:
             sys.exit(1)
 
     if 'sep' not in d:
-        print(__('''
-You have two options for placing the build directory for Sphinx output.
-Either, you use a directory "_build" within the root path, or you separate
-"source" and "build" directories within the root path.'''))
-        d['sep'] = do_prompt(__('Separate source and build directories (y/n)'),
-                             'n', boolean)
+        print()
+        print(__('You have two options for placing the build directory for Sphinx output.\n'
+                 'Either, you use a directory "_build" within the root path, or you separate\n'
+                 '"source" and "build" directories within the root path.'))
+        d['sep'] = do_prompt(__('Separate source and build directories (y/n)'), 'n', boolean)
 
     if 'dot' not in d:
-        print(__('''
-Inside the root directory, two more directories will be created; "_templates"
-for custom HTML templates and "_static" for custom stylesheets and other static
-files. You can enter another prefix (such as ".") to replace the underscore.'''))
+        print()
+        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # NOQA
+                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # NOQA
+                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # NOQA
         d['dot'] = do_prompt(__('Name prefix for templates and static dir'), '_', ok)
 
     if 'project' not in d:
-        print(__('''
-The project name will occur in several places in the built documentation.'''))
+        print()
+        print(__('The project name will occur in several places in the built documentation.'))
         d['project'] = do_prompt(__('Project name'))
     if 'author' not in d:
         d['author'] = do_prompt(__('Author name(s)'))
 
     if 'version' not in d:
-        print(__('''
-Sphinx has the notion of a "version" and a "release" for the
-software. Each version can have multiple releases. For example, for
-Python the version is something like 2.5 or 3.0, while the release is
-something like 2.5.1 or 3.0a1.  If you don't need this dual structure,
-just set both to the same value.'''))
+        print()
+        print(__('Sphinx has the notion of a "version" and a "release" for the\n'
+                 'software. Each version can have multiple releases. For example, for\n'
+                 'Python the version is something like 2.5 or 3.0, while the release is\n'
+                 'something like 2.5.1 or 3.0a1. If you don\'t need this dual structure,\n'
+                 'just set both to the same value.'))
         d['version'] = do_prompt(__('Project version'), '', allow_empty)
     if 'release' not in d:
         d['release'] = do_prompt(__('Project release'), d['version'], allow_empty)
 
     if 'language' not in d:
-        print(__('''
-If the documents are to be written in a language other than English,
-you can select a language here by its language code. Sphinx will then
-translate text that it generates into that language.
-
-For a list of supported codes, see
-https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.'''))
+        print()
+        print(__('If the documents are to be written in a language other than English,\n'
+                 'you can select a language here by its language code. Sphinx will then\n'
+                 'translate text that it generates into that language.\n'
+                 '\n'
+                 'For a list of supported codes, see\n'
+                 'https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.'))  # NOQA
         d['language'] = do_prompt(__('Project language'), 'en')
         if d['language'] == 'en':
             d['language'] = None
 
     if 'suffix' not in d:
-        print(__('''
-The file name suffix for source files. Commonly, this is either ".txt"
-or ".rst".  Only files with this suffix are considered documents.'''))
+        print()
+        print(__('The file name suffix for source files. Commonly, this is either ".txt"\n'
+                 'or ".rst". Only files with this suffix are considered documents.'))
         d['suffix'] = do_prompt(__('Source file suffix'), '.rst', suffix)
 
     if 'master' not in d:
-        print(__('''
-One document is special in that it is considered the top node of the
-"contents tree", that is, it is the root of the hierarchical structure
-of the documents. Normally, this is "index", but if your "index"
-document is a custom template, you can also set this to another filename.'''))
-        d['master'] = do_prompt(__('Name of your master document (without suffix)'),
-                                'index')
+        print()
+        print(__('One document is special in that it is considered the top node of the\n'
+                 '"contents tree", that is, it is the root of the hierarchical structure\n'
+                 'of the documents. Normally, this is "index", but if your "index"\n'
+                 'document is a custom template, you can also set this to another filename.'))
+        d['master'] = do_prompt(__('Name of your master document (without suffix)'), 'index')
 
     while path.isfile(path.join(d['path'], d['master'] + d['suffix'])) or \
             path.isfile(path.join(d['path'], 'source', d['master'] + d['suffix'])):
@@ -323,8 +294,7 @@ document is a custom template, you can also set this to another filename.'''))
                                    'existing file and press Enter'), d['master'])
 
     if 'extensions' not in d:
-        print(__('Indicate which of the following Sphinx extensions should be '
-                 'enabled:'))
+        print(__('Indicate which of the following Sphinx extensions should be enabled:'))
         d['extensions'] = []
         for name, description in EXTENSIONS.items():
             if do_prompt('%s: %s (y/n)' % (name, description), 'n', boolean):
@@ -332,20 +302,19 @@ document is a custom template, you can also set this to another filename.'''))
 
         # Handle conflicting options
         if {'sphinx.ext.imgmath', 'sphinx.ext.mathjax'}.issubset(d['extensions']):
-            print(__('Note: imgmath and mathjax cannot be enabled at the same '
-                     'time. imgmath has been deselected.'))
+            print(__('Note: imgmath and mathjax cannot be enabled at the same time. '
+                     'imgmath has been deselected.'))
             d['extensions'].remove('sphinx.ext.imgmath')
 
     if 'makefile' not in d:
-        print(__('''
-A Makefile and a Windows command file can be generated for you so that you
-only have to run e.g. `make html' instead of invoking sphinx-build
-directly.'''))
+        print()
+        print(__('A Makefile and a Windows command file can be generated for you so that you\n'
+                 'only have to run e.g. `make html\' instead of invoking sphinx-build\n'
+                 'directly.'))
         d['makefile'] = do_prompt(__('Create Makefile? (y/n)'), 'y', boolean)
 
     if 'batchfile' not in d:
-        d['batchfile'] = do_prompt(__('Create Windows command file? (y/n)'),
-                                   'y', boolean)
+        d['batchfile'] = do_prompt(__('Create Windows command file? (y/n)'), 'y', boolean)
     print()
 
 
@@ -359,14 +328,16 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     if 'mastertocmaxdepth' not in d:
         d['mastertocmaxdepth'] = 2
 
+    d['root_doc'] = d['master']
     d['now'] = time.asctime()
     d['project_underline'] = column_width(d['project']) * '='
     d.setdefault('extensions', [])
     d['copyright'] = time.strftime('%Y') + ', ' + d['author']
 
+    d["path"] = os.path.abspath(d['path'])
     ensuredir(d['path'])
 
-    srcdir = d['sep'] and path.join(d['path'], 'source') or d['path']
+    srcdir = path.join(d['path'], 'source') if d['sep'] else d['path']
 
     ensuredir(srcdir)
     if d['sep']:
@@ -396,13 +367,19 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     conf_path = os.path.join(templatedir, 'conf.py_t') if templatedir else None
     if not conf_path or not path.isfile(conf_path):
         conf_path = os.path.join(package_dir, 'templates', 'quickstart', 'conf.py_t')
-    with open(conf_path) as f:
+    with open(conf_path, encoding="utf-8") as f:
         conf_text = f.read()
 
     write_file(path.join(srcdir, 'conf.py'), template.render_string(conf_text, d))
 
     masterfile = path.join(srcdir, d['master'] + d['suffix'])
-    write_file(masterfile, template.render('quickstart/master_doc.rst_t', d))
+    if template._has_custom_template('quickstart/master_doc.rst_t'):
+        msg = ('A custom template `master_doc.rst_t` found. It has been renamed to '
+               '`root_doc.rst_t`.  Please rename it on your project too.')
+        print(colorize('red', msg))  # RemovedInSphinx60Warning
+        write_file(masterfile, template.render('quickstart/master_doc.rst_t', d))
+    else:
+        write_file(masterfile, template.render('quickstart/root_doc.rst_t', d))
 
     if d.get('make_mode') is True:
         makefile_template = 'quickstart/Makefile.new_t'
@@ -412,15 +389,15 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
         batchfile_template = 'quickstart/make.bat_t'
 
     if d['makefile'] is True:
-        d['rsrcdir'] = d['sep'] and 'source' or '.'
-        d['rbuilddir'] = d['sep'] and 'build' or d['dot'] + 'build'
+        d['rsrcdir'] = 'source' if d['sep'] else '.'
+        d['rbuilddir'] = 'build' if d['sep'] else d['dot'] + 'build'
         # use binary mode, to avoid writing \r\n on Windows
         write_file(path.join(d['path'], 'Makefile'),
                    template.render(makefile_template, d), '\n')
 
     if d['batchfile'] is True:
-        d['rsrcdir'] = d['sep'] and 'source' or '.'
-        d['rbuilddir'] = d['sep'] and 'build' or d['dot'] + 'build'
+        d['rsrcdir'] = 'source' if d['sep'] else '.'
+        d['rbuilddir'] = 'build' if d['sep'] else d['dot'] + 'build'
         write_file(path.join(d['path'], 'make.bat'),
                    template.render(batchfile_template, d), '\r\n')
 
@@ -428,17 +405,18 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
         return
     print()
     print(bold(__('Finished: An initial directory structure has been created.')))
-    print(__('''
-You should now populate your master file %s and create other documentation
-source files. ''') % masterfile + ((d['makefile'] or d['batchfile']) and __('''\
-Use the Makefile to build the docs, like so:
-   make builder
-''') or __('''\
-Use the sphinx-build command to build the docs, like so:
-   sphinx-build -b builder %s %s
-''') % (srcdir, builddir)) + __('''\
-where "builder" is one of the supported builders, e.g. html, latex or linkcheck.
-'''))
+    print()
+    print(__('You should now populate your master file %s and create other documentation\n'
+             'source files. ') % masterfile, end='')
+    if d['makefile'] or d['batchfile']:
+        print(__('Use the Makefile to build the docs, like so:\n'
+                 '   make builder'))
+    else:
+        print(__('Use the sphinx-build command to build the docs, like so:\n'
+                 '   sphinx-build -b builder %s %s') % (srcdir, builddir))
+    print(__('where "builder" is one of the supported builders, '
+             'e.g. html, latex or linkcheck.'))
+    print()
 
 
 def valid_dir(d: Dict) -> bool:
@@ -471,16 +449,18 @@ def valid_dir(d: Dict) -> bool:
 
 
 def get_parser() -> argparse.ArgumentParser:
+    description = __(
+        "\n"
+        "Generate required files for a Sphinx project.\n"
+        "\n"
+        "sphinx-quickstart is an interactive tool that asks some questions about your\n"
+        "project and then generates a complete documentation directory and sample\n"
+        "Makefile to be used with sphinx-build.\n"
+    )
     parser = argparse.ArgumentParser(
         usage='%(prog)s [OPTIONS] <PROJECT_DIR>',
-        epilog=__("For more information, visit <http://sphinx-doc.org/>."),
-        description=__("""
-Generate required files for a Sphinx project.
-
-sphinx-quickstart is an interactive tool that asks some questions about your
-project and then generates a complete documentation directory and sample
-Makefile to be used with sphinx-build.
-"""))
+        epilog=__("For more information, visit <https://www.sphinx-doc.org/>."),
+        description=description)
 
     parser.add_argument('-q', '--quiet', action='store_true', dest='quiet',
                         default=None,
@@ -492,8 +472,10 @@ Makefile to be used with sphinx-build.
                         help=__('project root'))
 
     group = parser.add_argument_group(__('Structure options'))
-    group.add_argument('--sep', action='store_true', default=None,
+    group.add_argument('--sep', action='store_true', dest='sep', default=None,
                        help=__('if specified, separate source and build dirs'))
+    group.add_argument('--no-sep', action='store_false', dest='sep',
+                       help=__('if specified, create build dir under source dir'))
     group.add_argument('--dot', metavar='DOT', default='_',
                        help=__('replacement for dot in _templates etc.'))
 
@@ -579,8 +561,8 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     try:
         if 'quiet' in d:
             if not {'project', 'author'}.issubset(d):
-                print(__('''"quiet" is specified, but any of "project" or \
-"author" is not specified.'''))
+                print(__('"quiet" is specified, but any of "project" or '
+                         '"author" is not specified.'))
                 return 1
 
         if {'quiet', 'project', 'author'}.issubset(d):
