@@ -1,29 +1,15 @@
-"""
-    sphinx.locale
-    ~~~~~~~~~~~~~
-
-    Locale utilities.
-
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Locale utilities."""
 
 import gettext
 import locale
-import warnings
 from collections import UserString, defaultdict
 from gettext import NullTranslations
-
-from sphinx.deprecation import RemovedInSphinx30Warning
-
-if False:
-    # For type annotation
-    from typing import Any, Callable, Dict, Iterable, List, Tuple, Union  # NOQA
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 
 class _TranslationProxy(UserString):
     """
-    Class for proxy strings from gettext translations.  This is a helper for the
+    Class for proxy strings from gettext translations. This is a helper for the
     lazy_* functions from this module.
 
     The proxy implementation attempts to be as complete as possible, so that
@@ -35,32 +21,27 @@ class _TranslationProxy(UserString):
     """
     __slots__ = ('_func', '_args')
 
-    def __new__(cls, func, *args):  # type: ignore
-        # type: (Callable, str) -> object
+    def __new__(cls, func: Callable, *args: str) -> object:  # type: ignore
         if not args:
             # not called with "function" and "arguments", but a plain string
             return str(func)
         return object.__new__(cls)
 
-    def __getnewargs__(self):
-        # type: () -> Tuple[str]
+    def __getnewargs__(self) -> Tuple[str]:
         return (self._func,) + self._args  # type: ignore
 
-    def __init__(self, func, *args):
-        # type: (Callable, str) -> None
+    def __init__(self, func: Callable, *args: str) -> None:
         self._func = func
         self._args = args
 
     @property
-    def data(self):  # type: ignore
-        # type: () -> str
+    def data(self) -> str:  # type: ignore
         return self._func(*self._args)
 
     # replace function from UserString; it instantiates a self.__class__
     # for the encoding result
 
-    def encode(self, encoding=None, errors=None):  # type: ignore
-        # type: (str, str) -> bytes
+    def encode(self, encoding: str = None, errors: str = None) -> bytes:  # type: ignore
         if encoding:
             if errors:
                 return self.data.encode(encoding, errors)
@@ -69,91 +50,58 @@ class _TranslationProxy(UserString):
         else:
             return self.data.encode()
 
-    def __dir__(self):
-        # type: () -> List[str]
+    def __dir__(self) -> List[str]:
         return dir(str)
 
-    def __str__(self):
-        # type: () -> str
+    def __str__(self) -> str:
         return str(self.data)
 
-    def __add__(self, other):  # type: ignore
-        # type: (str) -> str
+    def __add__(self, other: str) -> str:  # type: ignore
         return self.data + other
 
-    def __radd__(self, other):
-        # type: (str) -> str
+    def __radd__(self, other: str) -> str:  # type: ignore
         return other + self.data
 
-    def __mod__(self, other):  # type: ignore
-        # type: (str) -> str
+    def __mod__(self, other: str) -> str:  # type: ignore
         return self.data % other
 
-    def __rmod__(self, other):
-        # type: (str) -> str
+    def __rmod__(self, other: str) -> str:  # type: ignore
         return other % self.data
 
-    def __mul__(self, other):  # type: ignore
-        # type: (Any) -> str
+    def __mul__(self, other: Any) -> str:  # type: ignore
         return self.data * other
 
-    def __rmul__(self, other):
-        # type: (Any) -> str
+    def __rmul__(self, other: Any) -> str:  # type: ignore
         return other * self.data
 
-    def __getattr__(self, name):
-        # type: (str) -> Any
+    def __getattr__(self, name: str) -> Any:
         if name == '__members__':
             return self.__dir__()
         return getattr(self.data, name)
 
-    def __getstate__(self):
-        # type: () -> Tuple[Callable, Tuple[str, ...]]
+    def __getstate__(self) -> Tuple[Callable, Tuple[str, ...]]:
         return self._func, self._args
 
-    def __setstate__(self, tup):
-        # type: (Tuple[Callable, Tuple[str]]) -> None
+    def __setstate__(self, tup: Tuple[Callable, Tuple[str]]) -> None:
         self._func, self._args = tup
 
-    def __copy__(self):
-        # type: () -> _TranslationProxy
+    def __copy__(self) -> "_TranslationProxy":
         return self
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         try:
             return 'i' + repr(str(self.data))
         except Exception:
             return '<%s broken>' % self.__class__.__name__
 
 
-def mygettext(string):
-    # type: (str) -> str
-    """Used instead of _ when creating TranslationProxies, because _ is
-    not bound yet at that time.
-    """
-    warnings.warn('sphinx.locale.mygettext() is deprecated.  Please use `_()` instead.',
-                  RemovedInSphinx30Warning, stacklevel=2)
-    return _(string)
+translators: Dict[Tuple[str, str], NullTranslations] = defaultdict(NullTranslations)
 
 
-def lazy_gettext(string):
-    # type: (str) -> str
-    """A lazy version of `gettext`."""
-    # if isinstance(string, _TranslationProxy):
-    #     return string
-    warnings.warn('sphinx.locale.laxy_gettext() is deprecated.  Please use `_()` instead.',
-                  RemovedInSphinx30Warning, stacklevel=2)
-    return _TranslationProxy(mygettext, string)  # type: ignore
-
-
-translators = defaultdict(NullTranslations)  # type: Dict[Tuple[str, str], NullTranslations]
-
-
-def init(locale_dirs, language, catalog='sphinx', namespace='general'):
-    # type: (List[str], str, str, str) -> Tuple[NullTranslations, bool]
+def init(locale_dirs: List[Optional[str]], language: Optional[str],
+         catalog: str = 'sphinx', namespace: str = 'general') -> Tuple[NullTranslations, bool]:
     """Look for message catalogs in `locale_dirs` and *ensure* that there is at
-    least a NullTranslations catalog set in `translators`.  If called multiple
+    least a NullTranslations catalog set in `translators`. If called multiple
     times or if several ``.mo`` files are found, their contents are merged
     together (thus making ``init`` reentrant).
     """
@@ -167,9 +115,11 @@ def init(locale_dirs, language, catalog='sphinx', namespace='general'):
 
     if language and '_' in language:
         # for language having country code (like "de_AT")
-        languages = [language, language.split('_')[0]]
-    else:
+        languages: Optional[List[str]] = [language, language.split('_')[0]]
+    elif language:
         languages = [language]
+    else:
+        languages = None
 
     # loading
     for dir_ in locale_dirs:
@@ -190,8 +140,7 @@ def init(locale_dirs, language, catalog='sphinx', namespace='general'):
     return translator, has_translation
 
 
-def setlocale(category, value=None):
-    # type: (int, Union[str, Iterable[str]]) -> None
+def setlocale(category: int, value: Union[str, Iterable[str]] = None) -> None:
     """Update locale settings.
 
     This does not throw any exception even if update fails.
@@ -211,15 +160,14 @@ def setlocale(category, value=None):
         pass
 
 
-def init_console(locale_dir, catalog):
-    # type: (str, str) -> Tuple[NullTranslations, bool]
+def init_console(locale_dir: str, catalog: str) -> Tuple[NullTranslations, bool]:
     """Initialize locale for console.
 
     .. versionadded:: 1.8
     """
     try:
         # encoding is ignored
-        language, _ = locale.getlocale(locale.LC_MESSAGES)
+        language, _ = locale.getlocale(locale.LC_MESSAGES)  # type: Tuple[Optional[str], Any]
     except AttributeError:
         # LC_MESSAGES is not always defined. Fallback to the default language
         # in case it is not.
@@ -227,18 +175,15 @@ def init_console(locale_dir, catalog):
     return init([locale_dir], language, catalog, 'console')
 
 
-def get_translator(catalog='sphinx', namespace='general'):
-    # type: (str, str) -> NullTranslations
+def get_translator(catalog: str = 'sphinx', namespace: str = 'general') -> NullTranslations:
     return translators[(namespace, catalog)]
 
 
-def is_translator_registered(catalog='sphinx', namespace='general'):
-    # type: (str, str) -> bool
+def is_translator_registered(catalog: str = 'sphinx', namespace: str = 'general') -> bool:
     return (namespace, catalog) in translators
 
 
-def _lazy_translate(catalog, namespace, message):
-    # type: (str, str, str) -> str
+def _lazy_translate(catalog: str, namespace: str, message: str) -> str:
     """Used instead of _ when creating TranslationProxy, because _ is
     not bound yet at that time.
     """
@@ -246,7 +191,7 @@ def _lazy_translate(catalog, namespace, message):
     return translator.gettext(message)
 
 
-def get_translation(catalog, namespace='general'):
+def get_translation(catalog: str, namespace: str = 'general') -> Callable:
     """Get a translation function based on the *catalog* and *namespace*.
 
     The extension can use this API to translate the messages on the
@@ -261,7 +206,7 @@ def get_translation(catalog, namespace='general'):
 
 
         def setup(app):
-            package_dir = path.abspath(path.dirname(__file__))
+            package_dir = os.path.abspath(os.path.dirname(__file__))
             locale_dir = os.path.join(package_dir, 'locales')
             app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
 
@@ -271,8 +216,7 @@ def get_translation(catalog, namespace='general'):
 
     .. versionadded:: 1.8
     """
-    def gettext(message, *args):
-        # type: (str, *Any) -> str
+    def gettext(message: str, *args: Any) -> str:
         if not is_translator_registered(catalog, namespace):
             # not initialized yet
             return _TranslationProxy(_lazy_translate, catalog, namespace, message)  # type: ignore  # NOQA
@@ -295,12 +239,6 @@ _ = get_translation('sphinx')
 __ = get_translation('sphinx', 'console')
 
 
-def l_(*args):
-    warnings.warn('sphinx.locale.l_() is deprecated.  Please use `_()` instead.',
-                  RemovedInSphinx30Warning, stacklevel=2)
-    return _(*args)
-
-
 # labels
 admonitionlabels = {
     'attention': _('Attention'),
@@ -315,8 +253,8 @@ admonitionlabels = {
     'warning':   _('Warning'),
 }
 
-# Moved to sphinx.directives.other (will be overriden later)
-versionlabels = {}  # type: Dict[str, str]
+# Moved to sphinx.directives.other (will be overridden later)
+versionlabels: Dict[str, str] = {}
 
-# Moved to sphinx.domains.python (will be overriden later)
-pairindextypes = {}  # type: Dict[str, str]
+# Moved to sphinx.domains.python (will be overridden later)
+pairindextypes: Dict[str, str] = {}
