@@ -1,34 +1,21 @@
-"""
-    sphinx.roles
-    ~~~~~~~~~~~~
-
-    Handlers for additional ReST roles.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Handlers for additional ReST roles."""
 
 import re
-import warnings
-from typing import Any, Dict, List, Tuple
-from typing import Type  # for python3.5.1
+from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Type
 
+import docutils.parsers.rst.directives
+import docutils.parsers.rst.roles
+import docutils.parsers.rst.states
 from docutils import nodes, utils
 from docutils.nodes import Element, Node, TextElement, system_message
-from docutils.parsers.rst.states import Inliner
 
 from sphinx import addnodes
-from sphinx.deprecation import RemovedInSphinx40Warning
-from sphinx.locale import _
+from sphinx.locale import _, __
 from sphinx.util import ws_re
 from sphinx.util.docutils import ReferenceRole, SphinxRole
-from sphinx.util.nodes import (
-    split_explicit_title, process_index_entry, set_role_source_info
-)
 from sphinx.util.typing import RoleFunction
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
     from sphinx.environment import BuildEnvironment
 
@@ -73,8 +60,8 @@ class XRefRole(ReferenceRole):
     * Subclassing and overwriting `process_link()` and/or `result_nodes()`.
     """
 
-    nodeclass = addnodes.pending_xref   # type: Type[Element]
-    innernodeclass = nodes.literal      # type: Type[TextElement]
+    nodeclass: Type[Element] = addnodes.pending_xref
+    innernodeclass: Type[TextElement] = nodes.literal
 
     def __init__(self, fix_parens: bool = False, lowercase: bool = False,
                  nodeclass: Type[Element] = None, innernodeclass: Type[TextElement] = None,
@@ -89,22 +76,6 @@ class XRefRole(ReferenceRole):
 
         super().__init__()
 
-    def _fix_parens(self, env: "BuildEnvironment", has_explicit_title: bool, title: str,
-                    target: str) -> Tuple[str, str]:
-        warnings.warn('XRefRole._fix_parens() is deprecated.',
-                      RemovedInSphinx40Warning, stacklevel=2)
-        if not has_explicit_title:
-            if title.endswith('()'):
-                # remove parentheses
-                title = title[:-2]
-            if env.config.add_function_parentheses:
-                # add them back to all occurrences if configured
-                title += '()'
-        # remove parentheses from the target too
-        if target.endswith('()'):
-            target = target[:-2]
-        return title, target
-
     def update_title_and_target(self, title: str, target: str) -> Tuple[str, str]:
         if not self.has_explicit_title:
             if title.endswith('()'):
@@ -195,75 +166,6 @@ class AnyXRefRole(XRefRole):
         return result
 
 
-def indexmarkup_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
-                     options: Dict = {}, content: List[str] = []
-                     ) -> Tuple[List[Node], List[system_message]]:
-    """Role for PEP/RFC references that generate an index entry."""
-    warnings.warn('indexmarkup_role() is deprecated.  Please use PEP or RFC class instead.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    env = inliner.document.settings.env
-    if not typ:
-        assert env.temp_data['default_role']
-        typ = env.temp_data['default_role'].lower()
-    else:
-        typ = typ.lower()
-
-    has_explicit_title, title, target = split_explicit_title(text)
-    title = utils.unescape(title)
-    target = utils.unescape(target)
-    targetid = 'index-%s' % env.new_serialno('index')
-    indexnode = addnodes.index()
-    targetnode = nodes.target('', '', ids=[targetid])
-    inliner.document.note_explicit_target(targetnode)
-    if typ == 'pep':
-        indexnode['entries'] = [
-            ('single', _('Python Enhancement Proposals; PEP %s') % target,
-             targetid, '', None)]
-        anchor = ''
-        anchorindex = target.find('#')
-        if anchorindex > 0:
-            target, anchor = target[:anchorindex], target[anchorindex:]
-        if not has_explicit_title:
-            title = "PEP " + utils.unescape(title)
-        try:
-            pepnum = int(target)
-        except ValueError:
-            msg = inliner.reporter.error('invalid PEP number %s' % target,
-                                         line=lineno)
-            prb = inliner.problematic(rawtext, rawtext, msg)
-            return [prb], [msg]
-        ref = inliner.document.settings.pep_base_url + 'pep-%04d' % pepnum
-        sn = nodes.strong(title, title)
-        rn = nodes.reference('', '', internal=False, refuri=ref + anchor,
-                             classes=[typ])
-        rn += sn
-        return [indexnode, targetnode, rn], []
-    elif typ == 'rfc':
-        indexnode['entries'] = [
-            ('single', 'RFC; RFC %s' % target, targetid, '', None)]
-        anchor = ''
-        anchorindex = target.find('#')
-        if anchorindex > 0:
-            target, anchor = target[:anchorindex], target[anchorindex:]
-        if not has_explicit_title:
-            title = "RFC " + utils.unescape(title)
-        try:
-            rfcnum = int(target)
-        except ValueError:
-            msg = inliner.reporter.error('invalid RFC number %s' % target,
-                                         line=lineno)
-            prb = inliner.problematic(rawtext, rawtext, msg)
-            return [prb], [msg]
-        ref = inliner.document.settings.rfc_base_url + inliner.rfc_url % rfcnum
-        sn = nodes.strong(title, title)
-        rn = nodes.reference('', '', internal=False, refuri=ref + anchor,
-                             classes=[typ])
-        rn += sn
-        return [indexnode, targetnode, rn], []
-    else:
-        raise ValueError('unknown role type: %s' % typ)
-
-
 class PEP(ReferenceRole):
     def run(self) -> Tuple[List[Node], List[system_message]]:
         target_id = 'index-%s' % self.env.new_serialno('index')
@@ -283,7 +185,7 @@ class PEP(ReferenceRole):
                 title = "PEP " + self.title
                 reference += nodes.strong(title, title)
         except ValueError:
-            msg = self.inliner.reporter.error('invalid PEP number %s' % self.target,
+            msg = self.inliner.reporter.error(__('invalid PEP number %s') % self.target,
                                               line=self.lineno)
             prb = self.inliner.problematic(self.rawtext, self.rawtext, msg)
             return [prb], [msg]
@@ -294,9 +196,9 @@ class PEP(ReferenceRole):
         base_url = self.inliner.document.settings.pep_base_url
         ret = self.target.split('#', 1)
         if len(ret) == 2:
-            return base_url + 'pep-%04d#%s' % (int(ret[0]), ret[1])
+            return base_url + 'pep-%04d/#%s' % (int(ret[0]), ret[1])
         else:
-            return base_url + 'pep-%04d' % int(ret[0])
+            return base_url + 'pep-%04d/' % int(ret[0])
 
 
 class RFC(ReferenceRole):
@@ -317,7 +219,7 @@ class RFC(ReferenceRole):
                 title = "RFC " + self.title
                 reference += nodes.strong(title, title)
         except ValueError:
-            msg = self.inliner.reporter.error('invalid RFC number %s' % self.target,
+            msg = self.inliner.reporter.error(__('invalid RFC number %s') % self.target,
                                               line=self.lineno)
             prb = self.inliner.problematic(self.rawtext, self.rawtext, msg)
             return [prb], [msg]
@@ -336,44 +238,6 @@ class RFC(ReferenceRole):
 _amp_re = re.compile(r'(?<!&)&(?![&\s])')
 
 
-def menusel_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
-                 options: Dict = {}, content: List[str] = []
-                 ) -> Tuple[List[Node], List[system_message]]:
-    warnings.warn('menusel_role() is deprecated. '
-                  'Please use MenuSelection or GUILabel class instead.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    env = inliner.document.settings.env
-    if not typ:
-        assert env.temp_data['default_role']
-        typ = env.temp_data['default_role'].lower()
-    else:
-        typ = typ.lower()
-
-    text = utils.unescape(text)
-    if typ == 'menuselection':
-        text = text.replace('-->', '\N{TRIANGULAR BULLET}')
-    spans = _amp_re.split(text)
-
-    node = nodes.inline(rawtext=rawtext)
-    for i, span in enumerate(spans):
-        span = span.replace('&&', '&')
-        if i == 0:
-            if len(span) > 0:
-                textnode = nodes.Text(span)
-                node += textnode
-            continue
-        accel_node = nodes.inline()
-        letter_node = nodes.Text(span[0])
-        accel_node += letter_node
-        accel_node['classes'].append('accelerator')
-        node += accel_node
-        textnode = nodes.Text(span[1:])
-        node += textnode
-
-    node['classes'].append(typ)
-    return [node], []
-
-
 class GUILabel(SphinxRole):
     amp_re = re.compile(r'(?<!&)&(?![&\s])')
 
@@ -404,59 +268,6 @@ _litvar_re = re.compile('{([^}]+)}')
 parens_re = re.compile(r'(\\*{|\\*})')
 
 
-def emph_literal_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
-                      options: Dict = {}, content: List[str] = []
-                      ) -> Tuple[List[Node], List[system_message]]:
-    warnings.warn('emph_literal_role() is deprecated. '
-                  'Please use EmphasizedLiteral class instead.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    env = inliner.document.settings.env
-    if not typ:
-        assert env.temp_data['default_role']
-        typ = env.temp_data['default_role'].lower()
-    else:
-        typ = typ.lower()
-
-    retnode = nodes.literal(role=typ.lower(), classes=[typ])
-    parts = list(parens_re.split(utils.unescape(text)))
-    stack = ['']
-    for part in parts:
-        matched = parens_re.match(part)
-        if matched:
-            backslashes = len(part) - 1
-            if backslashes % 2 == 1:    # escaped
-                stack[-1] += "\\" * int((backslashes - 1) / 2) + part[-1]
-            elif part[-1] == '{':       # rparen
-                stack[-1] += "\\" * int(backslashes / 2)
-                if len(stack) >= 2 and stack[-2] == "{":
-                    # nested
-                    stack[-1] += "{"
-                else:
-                    # start emphasis
-                    stack.append('{')
-                    stack.append('')
-            else:                       # lparen
-                stack[-1] += "\\" * int(backslashes / 2)
-                if len(stack) == 3 and stack[1] == "{" and len(stack[2]) > 0:
-                    # emphasized word found
-                    if stack[0]:
-                        retnode += nodes.Text(stack[0], stack[0])
-                    retnode += nodes.emphasis(stack[2], stack[2])
-                    stack = ['']
-                else:
-                    # emphasized word not found; the rparen is not a special symbol
-                    stack.append('}')
-                    stack = [''.join(stack)]
-        else:
-            stack[-1] += part
-    if ''.join(stack):
-        # remaining is treated as Text
-        text = ''.join(stack)
-        retnode += nodes.Text(text, text)
-
-    return [retnode], []
-
-
 class EmphasizedLiteral(SphinxRole):
     parens_re = re.compile(r'(\\\\|\\{|\\}|{|})')
 
@@ -468,7 +279,7 @@ class EmphasizedLiteral(SphinxRole):
         return [node], []
 
     def parse(self, text: str) -> List[Node]:
-        result = []  # type: List[Node]
+        result: List[Node] = []
 
         stack = ['']
         for part in self.parens_re.split(text):
@@ -485,7 +296,7 @@ class EmphasizedLiteral(SphinxRole):
                 if len(stack) == 3 and stack[1] == "{" and len(stack[2]) > 0:
                     # emphasized word found
                     if stack[0]:
-                        result.append(nodes.Text(stack[0], stack[0]))
+                        result.append(nodes.Text(stack[0]))
                     result.append(nodes.emphasis(stack[2], stack[2]))
                     stack = ['']
                 else:
@@ -502,7 +313,7 @@ class EmphasizedLiteral(SphinxRole):
         if ''.join(stack):
             # remaining is treated as Text
             text = ''.join(stack)
-            result.append(nodes.Text(text, text))
+            result.append(nodes.Text(text))
 
         return result
 
@@ -510,93 +321,73 @@ class EmphasizedLiteral(SphinxRole):
 _abbr_re = re.compile(r'\((.*)\)$', re.S)
 
 
-def abbr_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
-              options: Dict = {}, content: List[str] = []
-              ) -> Tuple[List[Node], List[system_message]]:
-    warnings.warn('abbr_role() is deprecated.  Please use Abbrevation class instead.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    text = utils.unescape(text)
-    m = _abbr_re.search(text)
-    if m is None:
-        return [nodes.abbreviation(text, text, **options)], []
-    abbr = text[:m.start()].strip()
-    expl = m.group(1)
-    options = options.copy()
-    options['explanation'] = expl
-    return [nodes.abbreviation(abbr, abbr, **options)], []
-
-
 class Abbreviation(SphinxRole):
     abbr_re = re.compile(r'\((.*)\)$', re.S)
 
     def run(self) -> Tuple[List[Node], List[system_message]]:
+        options = self.options.copy()
         matched = self.abbr_re.search(self.text)
         if matched:
             text = self.text[:matched.start()].strip()
-            self.options['explanation'] = matched.group(1)
+            options['explanation'] = matched.group(1)
         else:
             text = self.text
 
-        return [nodes.abbreviation(self.rawtext, text, **self.options)], []
+        return [nodes.abbreviation(self.rawtext, text, **options)], []
 
 
-def index_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
-               options: Dict = {}, content: List[str] = []
-               ) -> Tuple[List[Node], List[system_message]]:
-    warnings.warn('index_role() is deprecated.  Please use Index class instead.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    # create new reference target
-    env = inliner.document.settings.env
-    targetid = 'index-%s' % env.new_serialno('index')
-    targetnode = nodes.target('', '', ids=[targetid])
-    # split text and target in role content
-    has_explicit_title, title, target = split_explicit_title(text)
-    title = utils.unescape(title)
-    target = utils.unescape(target)
-    # if an explicit target is given, we can process it as a full entry
-    if has_explicit_title:
-        entries = process_index_entry(target, targetid)
-    # otherwise we just create a "single" entry
-    else:
-        # but allow giving main entry
-        main = ''
-        if target.startswith('!'):
-            target = target[1:]
-            title = title[1:]
-            main = 'main'
-        entries = [('single', target, targetid, main, None)]
-    indexnode = addnodes.index()
-    indexnode['entries'] = entries
-    set_role_source_info(inliner, lineno, indexnode)
-    textnode = nodes.Text(title, title)
-    return [indexnode, targetnode, textnode], []
+# Sphinx provides the `code-block` directive for highlighting code blocks.
+# Docutils provides the `code` role which in theory can be used similarly by
+# defining a custom role for a given programming language:
+#
+#     .. .. role:: python(code)
+#          :language: python
+#          :class: highlight
+#
+# In practice this does not produce correct highlighting because it uses a
+# separate highlighting mechanism that results in the "long" pygments class
+# names rather than "short" pygments class names produced by the Sphinx
+# `code-block` directive and for which this extension contains CSS rules.
+#
+# In addition, even if that issue is fixed, because the highlighting
+# implementation in docutils, despite being based on pygments, differs from that
+# used by Sphinx, the output does not exactly match that produced by the Sphinx
+# `code-block` directive.
+#
+# This issue is noted here: //github.com/sphinx-doc/sphinx/issues/5157
+#
+# This overrides the docutils `code` role to perform highlighting in the same
+# way as the Sphinx `code-block` directive.
+#
+# TODO: Change to use `SphinxRole` once SphinxRole is fixed to support options.
+def code_role(name: str, rawtext: str, text: str, lineno: int,
+              inliner: docutils.parsers.rst.states.Inliner,
+              options: Dict = {}, content: List[str] = []
+              ) -> Tuple[List[Node], List[system_message]]:
+    options = options.copy()
+    docutils.parsers.rst.roles.set_classes(options)
+    language = options.get('language', '')
+    classes = ['code']
+    if language:
+        classes.append('highlight')
+    if 'classes' in options:
+        classes.extend(options['classes'])
+
+    if language and language not in classes:
+        classes.append(language)
+
+    node = nodes.literal(rawtext, text, classes=classes, language=language)
+
+    return [node], []
 
 
-class Index(ReferenceRole):
-    def run(self) -> Tuple[List[Node], List[system_message]]:
-        warnings.warn('Index role is deprecated.', RemovedInSphinx40Warning)
-        target_id = 'index-%s' % self.env.new_serialno('index')
-        if self.has_explicit_title:
-            # if an explicit target is given, process it as a full entry
-            title = self.title
-            entries = process_index_entry(self.target, target_id)
-        else:
-            # otherwise we just create a single entry
-            if self.target.startswith('!'):
-                title = self.title[1:]
-                entries = [('single', self.target[1:], target_id, 'main', None)]
-            else:
-                title = self.title
-                entries = [('single', self.target, target_id, '', None)]
-
-        index = addnodes.index(entries=entries)
-        target = nodes.target('', '', ids=[target_id])
-        text = nodes.Text(title, title)
-        self.set_source_info(index)
-        return [index, target, text], []
+code_role.options = {  # type: ignore
+    'class': docutils.parsers.rst.directives.class_option,
+    'language': docutils.parsers.rst.directives.unchanged,
+}
 
 
-specific_docroles = {
+specific_docroles: Dict[str, RoleFunction] = {
     # links to download references
     'download': XRefRole(nodeclass=addnodes.download_reference),
     # links to anything
@@ -609,7 +400,7 @@ specific_docroles = {
     'file': EmphasizedLiteral(),
     'samp': EmphasizedLiteral(),
     'abbr': Abbreviation(),
-}  # type: Dict[str, RoleFunction]
+}
 
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
@@ -623,6 +414,10 @@ def setup(app: "Sphinx") -> Dict[str, Any]:
     for rolename, func in specific_docroles.items():
         roles.register_local_role(rolename, func)
 
+    # Since docutils registers it as a canonical role, override it as a
+    # canonical role as well.
+    roles.register_canonical_role('code', code_role)
+
     return {
         'version': 'builtin',
         'parallel_read_safe': True,
