@@ -1,15 +1,7 @@
-"""
-    sphinx.util
-    ~~~~~~~~~~~
+"""Utility functions for Sphinx."""
 
-    Utility functions for Sphinx.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import fnmatch
 import functools
+import hashlib
 import os
 import posixpath
 import re
@@ -17,50 +9,36 @@ import sys
 import tempfile
 import traceback
 import unicodedata
-import warnings
-from codecs import BOM_UTF8
-from collections import deque
 from datetime import datetime
-from hashlib import md5
 from importlib import import_module
 from os import path
 from time import mktime, strptime
-from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Set, Tuple
-from urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
+from typing import (IO, TYPE_CHECKING, Any, Callable, Dict, Generator, Iterable, List,
+                    Optional, Pattern, Set, Tuple, Type, TypeVar)
+from urllib.parse import parse_qsl, quote_plus, urlencode, urlsplit, urlunsplit
 
-from docutils.utils import relative_path
-
-from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
-from sphinx.errors import PycodeError, SphinxParallelError, ExtensionError
+from sphinx.errors import ExtensionError, FiletypeNotFoundError, SphinxParallelError
 from sphinx.locale import __
 from sphinx.util import logging
-from sphinx.util.console import strip_colors, colorize, bold, term_width_line  # type: ignore
-from sphinx.util.fileutil import copy_asset_file
-from sphinx.util.typing import PathMatcher
-from sphinx.util import smartypants  # noqa
-
+from sphinx.util.console import bold, colorize, strip_colors, term_width_line  # type: ignore
+from sphinx.util.matching import patfilter  # noqa
+from sphinx.util.nodes import (caption_ref_re, explicit_title_re,  # noqa
+                               nested_parse_with_titles, split_explicit_title)
 # import other utilities; partly for backwards compatibility, so don't
 # prune unused ones indiscriminately
-from sphinx.util.osutil import (  # noqa
-    SEP, os_path, relative_uri, ensuredir, walk, mtimes_of_files, movefile,
-    copyfile, copytimes, make_filename, ustrftime)
-from sphinx.util.nodes import (   # noqa
-    nested_parse_with_titles, split_explicit_title, explicit_title_re,
-    caption_ref_re)
-from sphinx.util.matching import patfilter  # noqa
+from sphinx.util.osutil import (SEP, copyfile, copytimes, ensuredir, make_filename,  # noqa
+                                mtimes_of_files, os_path, relative_uri)
+from sphinx.util.typing import PathMatcher
 
-
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
-    from sphinx.builders import Builder
+
 
 logger = logging.getLogger(__name__)
 
 # Generally useful regular expressions.
-ws_re = re.compile(r'\s+')                      # type: Pattern
-url_re = re.compile(r'(?P<schema>.+)://.*')     # type: Pattern
+ws_re: Pattern = re.compile(r'\s+')
+url_re: Pattern = re.compile(r'(?P<schema>.+)://.*')
 
 
 # High-level utility functions.
@@ -71,7 +49,7 @@ def docname_join(basedocname: str, docname: str) -> str:
 
 
 def path_stabilize(filepath: str) -> str:
-    "normalize path separater and unicode string"
+    "Normalize path separator and unicode string"
     newpath = filepath.replace(os.path.sep, SEP)
     return unicodedata.normalize('NFC', newpath)
 
@@ -84,10 +62,11 @@ def get_matching_files(dirname: str,
     """
     # dirname is a normalized absolute path.
     dirname = path.normpath(path.abspath(dirname))
-    dirlen = len(dirname) + 1    # exclude final os.path.sep
 
     for root, dirs, files in os.walk(dirname, followlinks=True):
-        relativeroot = root[dirlen:]
+        relativeroot = path.relpath(root, dirname)
+        if relativeroot == ".":
+            relativeroot = ""  # suppress dirname for files on the target dir
 
         qdirs = enumerate(path_stabilize(path.join(relativeroot, dn))
                           for dn in dirs)  # type: Iterable[Tuple[int, str]]
@@ -99,25 +78,17 @@ def get_matching_files(dirname: str,
 
         dirs[:] = sorted(dirs[i] for (i, _) in qdirs)
 
-        for i, filename in sorted(qfiles):
+        for _i, filename in sorted(qfiles):
             yield filename
 
 
-def get_matching_docs(dirname: str, suffixes: List[str],
-                      exclude_matchers: Tuple[PathMatcher, ...] = ()) -> Iterable[str]:
-    """Get all file names (without suffixes) matching a suffix in a directory,
-    recursively.
-
-    Exclude files and dirs matching a pattern in *exclude_patterns*.
-    """
-    warnings.warn('get_matching_docs() is now deprecated. Use get_matching_files() instead.',
-                  RemovedInSphinx40Warning)
-    suffixpatterns = ['*' + s for s in suffixes]
-    for filename in get_matching_files(dirname, exclude_matchers):
-        for suffixpattern in suffixpatterns:
-            if fnmatch.fnmatch(filename, suffixpattern):
-                yield filename[:-len(suffixpattern) + 1]
-                break
+def get_filetype(source_suffix: Dict[str, str], filename: str) -> str:
+    for suffix, filetype in source_suffix.items():
+        if filename.endswith(suffix):
+            # If default filetype (None), considered as restructuredtext.
+            return filetype or 'restructuredtext'
+    else:
+        raise FiletypeNotFoundError
 
 
 class FilenameUniqDict(dict):
@@ -127,7 +98,7 @@ class FilenameUniqDict(dict):
     appear in.  Used for images and downloadable files in the environment.
     """
     def __init__(self) -> None:
-        self._existing = set()  # type: Set[str]
+        self._existing: Set[str] = set()
 
     def add_file(self, docname: str, newfile: str) -> str:
         if newfile in self:
@@ -151,7 +122,7 @@ class FilenameUniqDict(dict):
                 self._existing.discard(unique)
 
     def merge_other(self, docnames: Set[str], other: Dict[str, Tuple[Set[str], Any]]) -> None:
-        for filename, (docs, unique) in other.items():
+        for filename, (docs, _unique) in other.items():
             for doc in docs & set(docnames):
                 self.add_file(doc, filename)
 
@@ -162,6 +133,36 @@ class FilenameUniqDict(dict):
         self._existing = state
 
 
+def md5(data=b'', **kwargs):
+    """Wrapper around hashlib.md5
+
+    Attempt call with 'usedforsecurity=False' if we get a ValueError, which happens when
+    OpenSSL FIPS mode is enabled:
+    ValueError: error:060800A3:digital envelope routines:EVP_DigestInit_ex:disabled for fips
+
+    See: https://github.com/sphinx-doc/sphinx/issues/7611
+    """
+
+    try:
+        return hashlib.md5(data, **kwargs)
+    except ValueError:
+        return hashlib.md5(data, **kwargs, usedforsecurity=False)  # type: ignore
+
+
+def sha1(data=b'', **kwargs):
+    """Wrapper around hashlib.sha1
+
+    Attempt call with 'usedforsecurity=False' if we get a ValueError
+
+    See: https://github.com/sphinx-doc/sphinx/issues/7611
+    """
+
+    try:
+        return hashlib.sha1(data, **kwargs)
+    except ValueError:
+        return hashlib.sha1(data, **kwargs, usedforsecurity=False)  # type: ignore
+
+
 class DownloadFiles(dict):
     """A special dictionary for download files.
 
@@ -179,46 +180,17 @@ class DownloadFiles(dict):
         return self[filename][1]
 
     def purge_doc(self, docname: str) -> None:
-        for filename, (docs, dest) in list(self.items()):
+        for filename, (docs, _dest) in list(self.items()):
             docs.discard(docname)
             if not docs:
                 del self[filename]
 
     def merge_other(self, docnames: Set[str], other: Dict[str, Tuple[Set[str], Any]]) -> None:
-        for filename, (docs, dest) in other.items():
+        for filename, (docs, _dest) in other.items():
             for docname in docs & set(docnames):
                 self.add_file(docname, filename)
 
 
-def copy_static_entry(source: str, targetdir: str, builder: "Builder", context: Dict = {},
-                      exclude_matchers: Tuple[PathMatcher, ...] = (), level: int = 0) -> None:
-    """[DEPRECATED] Copy a HTML builder static_path entry from source to targetdir.
-
-    Handles all possible cases of files, directories and subdirectories.
-    """
-    warnings.warn('sphinx.util.copy_static_entry is deprecated for removal',
-                  RemovedInSphinx30Warning, stacklevel=2)
-
-    if exclude_matchers:
-        relpath = relative_path(path.join(builder.srcdir, 'dummy'), source)
-        for matcher in exclude_matchers:
-            if matcher(relpath):
-                return
-    if path.isfile(source):
-        copy_asset_file(source, targetdir, context, builder.templates)
-    elif path.isdir(source):
-        ensuredir(targetdir)
-        for entry in os.listdir(source):
-            if entry.startswith('.'):
-                continue
-            newtarget = targetdir
-            if path.isdir(path.join(source, entry)):
-                newtarget = path.join(targetdir, entry)
-            copy_static_entry(path.join(source, entry), newtarget,
-                              builder, context, level=level + 1,
-                              exclude_matchers=exclude_matchers)
-
-
 _DEBUG_HEADER = '''\
 # Sphinx version: %s
 # Python version: %s (%s)
@@ -232,10 +204,12 @@ _DEBUG_HEADER = '''\
 
 def save_traceback(app: "Sphinx") -> str:
     """Save the current exception's traceback in a temporary file."""
-    import sphinx
-    import jinja2
-    import docutils
     import platform
+
+    import docutils
+    import jinja2
+
+    import sphinx
     exc = sys.exc_info()[1]
     if isinstance(exc, SphinxParallelError):
         exc_format = '(Error in parallel process)\n' + exc.traceback
@@ -252,7 +226,7 @@ def save_traceback(app: "Sphinx") -> str:
                    platform.python_version(),
                    platform.python_implementation(),
                    docutils.__version__, docutils.__version_details__,
-                   jinja2.__version__,  # type: ignore
+                   jinja2.__version__,
                    last_msgs)).encode())
     if app is not None:
         for ext in app.extensions.values():
@@ -265,51 +239,6 @@ def save_traceback(app: "Sphinx") -> str:
     return path
 
 
-def get_module_source(modname: str) -> Tuple[str, str]:
-    """Try to find the source code for a module.
-
-    Can return ('file', 'filename') in which case the source is in the given
-    file, or ('string', 'source') which which case the source is the string.
-    """
-    try:
-        mod = import_module(modname)
-    except Exception as err:
-        raise PycodeError('error importing %r' % modname, err)
-    filename = getattr(mod, '__file__', None)
-    loader = getattr(mod, '__loader__', None)
-    if loader and getattr(loader, 'get_filename', None):
-        try:
-            filename = loader.get_filename(modname)
-        except Exception as err:
-            raise PycodeError('error getting filename for %r' % filename, err)
-    if filename is None and loader:
-        try:
-            filename = loader.get_source(modname)
-            if filename:
-                return 'string', filename
-        except Exception as err:
-            raise PycodeError('error getting source for %r' % modname, err)
-    if filename is None:
-        raise PycodeError('no source found for module %r' % modname)
-    filename = path.normpath(path.abspath(filename))
-    lfilename = filename.lower()
-    if lfilename.endswith('.pyo') or lfilename.endswith('.pyc'):
-        filename = filename[:-1]
-        if not path.isfile(filename) and path.isfile(filename + 'w'):
-            filename += 'w'
-    elif not (lfilename.endswith('.py') or lfilename.endswith('.pyw')):
-        raise PycodeError('source is not a .py file: %r' % filename)
-    elif ('.egg' + os.path.sep) in filename:
-        pat = '(?<=\\.egg)' + re.escape(os.path.sep)
-        eggpath, _ = re.split(pat, filename, 1)
-        if path.isfile(eggpath):
-            return 'file', filename
-
-    if not path.isfile(filename):
-        raise PycodeError('source file is not present: %r' % filename)
-    return 'file', filename
-
-
 def get_full_modname(modname: str, attribute: str) -> str:
     if modname is None:
         # Prevents a TypeError: if the last getattr() call will return None
@@ -317,7 +246,7 @@ def get_full_modname(modname: str, attribute: str) -> str:
         return None
     module = import_module(modname)
 
-    # Allow an attribute to have multiple parts and incidentially allow
+    # Allow an attribute to have multiple parts and incidentally allow
     # repeated .s in the attribute.
     value = module
     for attr in attribute.split('.'):
@@ -331,56 +260,6 @@ def get_full_modname(modname: str, attribute: str) -> str:
 _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 
 
-def detect_encoding(readline: Callable[[], bytes]) -> str:
-    """Like tokenize.detect_encoding() from Py3k, but a bit simplified."""
-
-    def read_or_stop() -> bytes:
-        try:
-            return readline()
-        except StopIteration:
-            return None
-
-    def get_normal_name(orig_enc: str) -> str:
-        """Imitates get_normal_name in tokenizer.c."""
-        # Only care about the first 12 characters.
-        enc = orig_enc[:12].lower().replace('_', '-')
-        if enc == 'utf-8' or enc.startswith('utf-8-'):
-            return 'utf-8'
-        if enc in ('latin-1', 'iso-8859-1', 'iso-latin-1') or \
-           enc.startswith(('latin-1-', 'iso-8859-1-', 'iso-latin-1-')):
-            return 'iso-8859-1'
-        return orig_enc
-
-    def find_cookie(line: bytes) -> str:
-        try:
-            line_string = line.decode('ascii')
-        except UnicodeDecodeError:
-            return None
-
-        matches = _coding_re.findall(line_string)
-        if not matches:
-            return None
-        return get_normal_name(matches[0])
-
-    default = sys.getdefaultencoding()
-    first = read_or_stop()
-    if first and first.startswith(BOM_UTF8):
-        first = first[3:]
-        default = 'utf-8-sig'
-    if not first:
-        return default
-    encoding = find_cookie(first)
-    if encoding:
-        return encoding
-    second = read_or_stop()
-    if not second:
-        return default
-    encoding = find_cookie(second)
-    if encoding:
-        return encoding
-    return default
-
-
 class UnicodeDecodeErrorHandler:
     """Custom error handler for open() that warns and replaces."""
 
@@ -443,53 +322,12 @@ def parselinenos(spec: str, total: int) -> List[int]:
                 items.extend(range(start - 1, end))
             else:
                 raise ValueError
-        except Exception:
-            raise ValueError('invalid line number spec: %r' % spec)
+        except Exception as exc:
+            raise ValueError('invalid line number spec: %r' % spec) from exc
 
     return items
 
 
-def force_decode(string: str, encoding: str) -> str:
-    """Forcibly get a unicode string out of a bytestring."""
-    warnings.warn('force_decode() is deprecated.',
-                  RemovedInSphinx40Warning, stacklevel=2)
-    if isinstance(string, bytes):
-        try:
-            if encoding:
-                string = string.decode(encoding)
-            else:
-                # try decoding with utf-8, should only work for real UTF-8
-                string = string.decode()
-        except UnicodeError:
-            # last resort -- can't fail
-            string = string.decode('latin1')
-    return string
-
-
-class attrdict(dict):
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
-        warnings.warn('The attrdict class is deprecated.',
-                      RemovedInSphinx40Warning, stacklevel=2)
-
-    def __getattr__(self, key: str) -> str:
-        return self[key]
-
-    def __setattr__(self, key: str, val: str) -> None:
-        self[key] = val
-
-    def __delattr__(self, key: str) -> None:
-        del self[key]
-
-
-def rpartition(s: str, t: str) -> Tuple[str, str]:
-    """Similar to str.rpartition from 2.5, but doesn't return the separator."""
-    i = s.rfind(t)
-    if i != -1:
-        return s[:i], s[i + len(t):]
-    return '', s
-
-
 def split_into(n: int, type: str, value: str) -> List[str]:
     """Split an index entry into a given number of parts at semicolons."""
     parts = [x.strip() for x in value.split(';', n - 1)]
@@ -523,48 +361,13 @@ def format_exception_cut_frames(x: int = 1) -> str:
     """Format an exception with traceback, but only the last x frames."""
     typ, val, tb = sys.exc_info()
     # res = ['Traceback (most recent call last):\n']
-    res = []  # type: List[str]
+    res: List[str] = []
     tbres = traceback.format_tb(tb)
     res += tbres[-x:]
     res += traceback.format_exception_only(typ, val)
     return ''.join(res)
 
 
-class PeekableIterator:
-    """
-    An iterator which wraps any iterable and makes it possible to peek to see
-    what's the next item.
-    """
-    def __init__(self, iterable: Iterable) -> None:
-        self.remaining = deque()  # type: deque
-        self._iterator = iter(iterable)
-        warnings.warn('PeekableIterator is deprecated.',
-                      RemovedInSphinx40Warning, stacklevel=2)
-
-    def __iter__(self) -> "PeekableIterator":
-        return self
-
-    def __next__(self) -> Any:
-        """Return the next item from the iterator."""
-        if self.remaining:
-            return self.remaining.popleft()
-        return next(self._iterator)
-
-    next = __next__  # Python 2 compatibility
-
-    def push(self, item: Any) -> None:
-        """Push the `item` on the internal stack, it will be returned on the
-        next :meth:`next` call.
-        """
-        self.remaining.append(item)
-
-    def peek(self) -> Any:
-        """Return the next item without changing the state of the iterator."""
-        item = next(self)
-        self.push(item)
-        return item
-
-
 def import_object(objname: str, source: str = None) -> Any:
     """Import python object by qualname."""
     try:
@@ -582,9 +385,39 @@ def import_object(objname: str, source: str = None) -> Any:
     except (AttributeError, ImportError) as exc:
         if source:
             raise ExtensionError('Could not import %s (needed for %s)' %
-                                 (objname, source), exc)
+                                 (objname, source), exc) from exc
         else:
-            raise ExtensionError('Could not import %s' % objname, exc)
+            raise ExtensionError('Could not import %s' % objname, exc) from exc
+
+
+def split_full_qualified_name(name: str) -> Tuple[Optional[str], str]:
+    """Split full qualified name to a pair of modname and qualname.
+
+    A qualname is an abbreviation for "Qualified name" introduced at PEP-3155
+    (https://peps.python.org/pep-3155/).  It is a dotted path name
+    from the module top-level.
+
+    A "full" qualified name means a string containing both module name and
+    qualified name.
+
+    .. note:: This function actually imports the module to check its existence.
+              Therefore you need to mock 3rd party modules if needed before
+              calling this function.
+    """
+    parts = name.split('.')
+    for i, _part in enumerate(parts, 1):
+        try:
+            modname = ".".join(parts[:i])
+            import_module(modname)
+        except ImportError:
+            if parts[:i - 1]:
+                return ".".join(parts[:i - 1]), ".".join(parts[i - 1:])
+            else:
+                return None, ".".join(parts)
+        except IndexError:
+            pass
+
+    return name, ""
 
 
 def encode_uri(uri: str) -> str:
@@ -596,6 +429,14 @@ def encode_uri(uri: str) -> str:
     return urlunsplit(split)
 
 
+def isurl(url: str) -> bool:
+    """Check *url* is URL or not."""
+    if url and '://' in url:
+        return True
+    else:
+        return False
+
+
 def display_chunk(chunk: Any) -> str:
     if isinstance(chunk, (list, tuple)):
         if len(chunk) == 1:
@@ -604,8 +445,12 @@ def display_chunk(chunk: Any) -> str:
     return str(chunk)
 
 
-def old_status_iterator(iterable: Iterable, summary: str, color: str = "darkgreen",
-                        stringify_func: Callable[[Any], str] = display_chunk) -> Iterator:
+T = TypeVar('T')
+
+
+def old_status_iterator(iterable: Iterable[T], summary: str, color: str = "darkgreen",
+                        stringify_func: Callable[[Any], str] = display_chunk
+                        ) -> Generator[T, None, None]:
     l = 0
     for item in iterable:
         if l == 0:
@@ -619,9 +464,10 @@ def old_status_iterator(iterable: Iterable, summary: str, color: str = "darkgree
 
 
 # new version with progress info
-def status_iterator(iterable: Iterable, summary: str, color: str = "darkgreen",
+def status_iterator(iterable: Iterable[T], summary: str, color: str = "darkgreen",
                     length: int = 0, verbosity: int = 0,
-                    stringify_func: Callable[[Any], str] = display_chunk) -> Iterable:
+                    stringify_func: Callable[[Any], str] = display_chunk
+                    ) -> Generator[T, None, None]:
     if length == 0:
         yield from old_status_iterator(iterable, summary, color, stringify_func)
         return
@@ -651,7 +497,7 @@ class progress_message:
     def __enter__(self) -> None:
         logger.info(bold(self.message + '... '), nonl=True)
 
-    def __exit__(self, exc_type: "Type[Exception]", exc_value: Exception, traceback: Any) -> bool:  # NOQA
+    def __exit__(self, exc_type: Type[Exception], exc_value: Exception, traceback: Any) -> bool:  # NOQA
         if isinstance(exc_value, SkipProgressMessage):
             logger.info(__('skipped'))
             if exc_value.args:
@@ -666,7 +512,7 @@ class progress_message:
 
     def __call__(self, f: Callable) -> Callable:
         @functools.wraps(f)
-        def wrapper(*args, **kwargs):
+        def wrapper(*args: Any, **kwargs: Any) -> Any:
             with self:
                 return f(*args, **kwargs)
 
