@@ -1,24 +1,36 @@
-"""
-    test_build_gettext
-    ~~~~~~~~~~~~~~~~~~
-
-    Test the build process with gettext builder with the test root.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Test the build process with gettext builder with the test root."""
 
 import gettext
 import os
 import re
 import subprocess
-from subprocess import CalledProcessError, PIPE
+from subprocess import PIPE, CalledProcessError
 
 import pytest
 
+from sphinx.builders.gettext import Catalog, MsgOrigin
 from sphinx.util.osutil import cd
 
 
+def test_Catalog_duplicated_message():
+    catalog = Catalog()
+    catalog.add('hello', MsgOrigin('/path/to/filename', 1))
+    catalog.add('hello', MsgOrigin('/path/to/filename', 1))
+    catalog.add('hello', MsgOrigin('/path/to/filename', 2))
+    catalog.add('hello', MsgOrigin('/path/to/yetanother', 1))
+    catalog.add('world', MsgOrigin('/path/to/filename', 1))
+
+    assert len(list(catalog)) == 2
+
+    msg1, msg2 = list(catalog)
+    assert msg1.text == 'hello'
+    assert msg1.locations == [('/path/to/filename', 1),
+                              ('/path/to/filename', 2),
+                              ('/path/to/yetanother', 1)]
+    assert msg2.text == 'world'
+    assert msg2.locations == [('/path/to/filename', 1)]
+
+
 @pytest.mark.sphinx('gettext', srcdir='root-gettext')
 def test_build_gettext(app):
     # Generic build; should fail only when the builder is horribly broken.
@@ -31,7 +43,7 @@ def test_build_gettext(app):
     assert (app.outdir / 'subdir.pot').isfile()
 
     # regression test for issue #960
-    catalog = (app.outdir / 'markup.pot').text()
+    catalog = (app.outdir / 'markup.pot').read_text(encoding='utf8')
     assert 'msgid "something, something else, something more"' in catalog
 
 
@@ -84,7 +96,7 @@ def test_gettext_index_entries(app):
             return m.groups()[0]
         return None
 
-    pot = (app.outdir / 'index_entries.pot').text()
+    pot = (app.outdir / 'index_entries.pot').read_text(encoding='utf8')
     msgids = [_f for _f in map(msgid_getter, pot.splitlines()) if _f]
 
     expected_msgids = [
@@ -133,7 +145,7 @@ def test_gettext_disable_index_entries(app):
             return m.groups()[0]
         return None
 
-    pot = (app.outdir / 'index_entries.pot').text()
+    pot = (app.outdir / 'index_entries.pot').read_text(encoding='utf8')
     msgids = [_f for _f in map(msgid_getter, pot.splitlines()) if _f]
 
     expected_msgids = [
@@ -156,7 +168,7 @@ def test_gettext_template(app):
     app.builder.build_all()
     assert (app.outdir / 'sphinx.pot').isfile()
 
-    result = (app.outdir / 'sphinx.pot').text()
+    result = (app.outdir / 'sphinx.pot').read_text(encoding='utf8')
     assert "Welcome" in result
     assert "Sphinx %(version)s" in result
 
@@ -166,7 +178,7 @@ def test_gettext_template_msgid_order_in_sphinxpot(app):
     app.builder.build_all()
     assert (app.outdir / 'sphinx.pot').isfile()
 
-    result = (app.outdir / 'sphinx.pot').text()
+    result = (app.outdir / 'sphinx.pot').read_text(encoding='utf8')
     assert re.search(
         ('msgid "Template 1".*'
          'msgid "This is Template 1\\.".*'
@@ -174,3 +186,21 @@ def test_gettext_template_msgid_order_in_sphinxpot(app):
          'msgid "This is Template 2\\.".*'),
         result,
         flags=re.S)
+
+
+@pytest.mark.sphinx(
+    'gettext', srcdir='root-gettext',
+    confoverrides={'gettext_compact': 'documentation'})
+def test_build_single_pot(app):
+    app.builder.build_all()
+
+    assert (app.outdir / 'documentation.pot').isfile()
+
+    result = (app.outdir / 'documentation.pot').read_text(encoding='utf8')
+    assert re.search(
+        ('msgid "Todo".*'
+         'msgid "Like footnotes.".*'
+         'msgid "The minute.".*'
+         'msgid "Generated section".*'),
+        result,
+        flags=re.S)
