@@ -1,12 +1,4 @@
-"""
-    test_ext_math
-    ~~~~~~~~~~~~~
-
-    Test math extensions.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Test math extensions."""
 
 import re
 import subprocess
@@ -40,7 +32,7 @@ def test_imgmath_png(app, status, warning):
     if "dvipng command 'dvipng' cannot be run" in warning.getvalue():
         raise pytest.skip.Exception('dvipng command "dvipng" is not available')
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     html = (r'<div class="math">\s*<p>\s*<img src="_images/math/\w+.png"'
             r'\s*alt="a\^2\+b\^2=c\^2"/>\s*</p>\s*</div>')
     assert re.search(html, content, re.S)
@@ -58,7 +50,7 @@ def test_imgmath_svg(app, status, warning):
     if "dvisvgm command 'dvisvgm' cannot be run" in warning.getvalue():
         raise pytest.skip.Exception('dvisvgm command "dvisvgm" is not available')
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     html = (r'<div class="math">\s*<p>\s*<img src="_images/math/\w+.svg"'
             r'\s*alt="a\^2\+b\^2=c\^2"/>\s*</p>\s*</div>')
     assert re.search(html, content, re.S)
@@ -70,8 +62,8 @@ def test_imgmath_svg(app, status, warning):
 def test_mathjax_options(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
-    assert ('<script defer="defer" integrity="sha384-0123456789" '
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert ('<script async="async" integrity="sha384-0123456789" '
             'src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js">'
             '</script>' in content)
 
@@ -81,7 +73,7 @@ def test_mathjax_options(app, status, warning):
 def test_mathjax_align(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     html = (r'<div class="math notranslate nohighlight">\s*'
             r'\\\[ \\begin\{align\}\\begin\{aligned\}S \&amp;= \\pi r\^2\\\\'
             r'V \&amp;= \\frac\{4\}\{3\} \\pi r\^3\\end\{aligned\}\\end\{align\} \\\]</div>')
@@ -94,7 +86,7 @@ def test_mathjax_align(app, status, warning):
 def test_math_number_all_mathjax(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     html = (r'<div class="math notranslate nohighlight" id="equation-index-0">\s*'
             r'<span class="eqno">\(1\)<a .*>\xb6</a></span>\\\[a\^2\+b\^2=c\^2\\\]</div>')
     assert re.search(html, content, re.S)
@@ -105,7 +97,7 @@ def test_math_number_all_mathjax(app, status, warning):
 def test_math_number_all_latex(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'python.tex').read_text()
+    content = (app.outdir / 'python.tex').read_text(encoding='utf8')
     macro = (r'\\begin{equation\*}\s*'
              r'\\begin{split}a\^2\+b\^2=c\^2\\end{split}\s*'
              r'\\end{equation\*}')
@@ -135,7 +127,7 @@ def test_math_number_all_latex(app, status, warning):
 def test_math_eqref_format_html(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'math.html').read_text()
+    content = (app.outdir / 'math.html').read_text(encoding='utf8')
     html = ('<p>Referencing equation <a class="reference internal" '
             'href="#equation-foo">Eq.1</a> and <a class="reference internal" '
             'href="#equation-foo">Eq.1</a>.</p>')
@@ -148,7 +140,7 @@ def test_math_eqref_format_html(app, status, warning):
 def test_math_eqref_format_latex(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'python.tex').read_text()
+    content = (app.outdir / 'python.tex').read_text(encoding='utf8')
     macro = (r'Referencing equation Eq.\\ref{equation:math:foo} and '
              r'Eq.\\ref{equation:math:foo}.')
     assert re.search(macro, content, re.S)
@@ -161,7 +153,7 @@ def test_math_eqref_format_latex(app, status, warning):
 def test_mathjax_numfig_html(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'math.html').read_text()
+    content = (app.outdir / 'math.html').read_text(encoding='utf8')
     html = ('<div class="math notranslate nohighlight" id="equation-math-0">\n'
             '<span class="eqno">(1.2)')
     assert html in content
@@ -179,7 +171,7 @@ def test_mathjax_numfig_html(app, status, warning):
 def test_imgmath_numfig_html(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'page.html').read_text()
+    content = (app.outdir / 'page.html').read_text(encoding='utf8')
     html = '<span class="eqno">(3)<a class="headerlink" href="#equation-bar"'
     assert html in content
     html = ('<p>Referencing equations <a class="reference internal" '
@@ -219,8 +211,9 @@ def test_math_compat(app, status, warning):
 def test_mathjax3_config(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
+    assert ('<script defer="defer" src="%s">' % MATHJAX_URL in content)
     assert ('<script>window.MathJax = {"extensions": ["tex2jax.js"]}</script>' in content)
 
 
@@ -230,22 +223,45 @@ def test_mathjax3_config(app, status, warning):
 def test_mathjax2_config(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
-    assert MATHJAX_URL in content
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert ('<script async="async" src="%s">' % MATHJAX_URL in content)
     assert ('<script type="text/x-mathjax-config">'
             'MathJax.Hub.Config({"extensions": ["tex2jax.js"]})'
             '</script>' in content)
 
 
+@pytest.mark.sphinx('html', testroot='ext-math',
+                    confoverrides={'extensions': ['sphinx.ext.mathjax'],
+                                   'mathjax_options': {'async': 'async'},
+                                   'mathjax3_config': {'extensions': ['tex2jax.js']}})
+def test_mathjax_options_async_for_mathjax3(app, status, warning):
+    app.builder.build_all()
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert MATHJAX_URL in content
+    assert ('<script async="async" src="%s">' % MATHJAX_URL in content)
+
+
+@pytest.mark.sphinx('html', testroot='ext-math',
+                    confoverrides={'extensions': ['sphinx.ext.mathjax'],
+                                   'mathjax_options': {'defer': 'defer'},
+                                   'mathjax2_config': {'extensions': ['tex2jax.js']}})
+def test_mathjax_options_defer_for_mathjax2(app, status, warning):
+    app.builder.build_all()
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert ('<script defer="defer" src="%s">' % MATHJAX_URL in content)
+
+
 @pytest.mark.sphinx('html', testroot='ext-math',
                     confoverrides={'extensions': ['sphinx.ext.mathjax']})
 def test_mathjax_is_installed_only_if_document_having_math(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
 
-    content = (app.outdir / 'nomath.html').read_text()
+    content = (app.outdir / 'nomath.html').read_text(encoding='utf8')
     assert MATHJAX_URL not in content
 
 
@@ -254,7 +270,7 @@ def test_mathjax_is_installed_only_if_document_having_math(app, status, warning)
 def test_mathjax_is_not_installed_if_no_equations(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert 'MathJax.js' not in content
 
 
@@ -264,8 +280,8 @@ def test_mathjax_is_installed_if_no_equations_when_forced(app, status, warning):
     app.set_html_assets_policy('always')
     app.builder.build_all()
 
-    content = (app.outdir / 'index.html').read_text()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
 
-    content = (app.outdir / 'nomath.html').read_text()
+    content = (app.outdir / 'nomath.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
