@@ -1,41 +1,41 @@
-"""
-    test_markup
-    ~~~~~~~~~~~
-
-    Test various Sphinx-specific markup extensions.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Test various Sphinx-specific markup extensions."""
 
 import re
+import warnings
 
+import docutils
 import pytest
-from docutils import frontend, utils, nodes
+from docutils import frontend, nodes, utils
 from docutils.parsers.rst import Parser as RstParser
-from docutils.transforms.universal import SmartQuotes
 
 from sphinx import addnodes
+from sphinx.builders.html.transforms import KeyboardTransform
 from sphinx.builders.latex import LaTeXBuilder
+from sphinx.environment import default_settings
 from sphinx.roles import XRefRole
 from sphinx.testing.util import Struct, assert_node
+from sphinx.transforms import SphinxSmartQuotes
 from sphinx.util import texescape
 from sphinx.util.docutils import sphinx_domains
-from sphinx.writers.html import HTMLWriter, HTMLTranslator
-from sphinx.writers.latex import LaTeXWriter, LaTeXTranslator
+from sphinx.writers.html import HTMLTranslator, HTMLWriter
+from sphinx.writers.latex import LaTeXTranslator, LaTeXWriter
 
 
 @pytest.fixture
 def settings(app):
     texescape.init()  # otherwise done by the latex builder
-    optparser = frontend.OptionParser(
-        components=(RstParser, HTMLWriter, LaTeXWriter))
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', category=DeprecationWarning)
+        # DeprecationWarning: The frontend.OptionParser class will be replaced
+        # by a subclass of argparse.ArgumentParser in Docutils 0.21 or later.
+        optparser = frontend.OptionParser(
+            components=(RstParser, HTMLWriter, LaTeXWriter),
+            defaults=default_settings)
     settings = optparser.get_default_values()
     settings.smart_quotes = True
     settings.env = app.builder.env
     settings.env.temp_data['docname'] = 'dummy'
     settings.contentsname = 'dummy'
-    settings.rfc_base_url = 'http://tools.ietf.org/html/'
     domain_context = sphinx_domains(settings.env)
     domain_context.enable()
     yield settings
@@ -65,8 +65,8 @@ def parse(new_document):
         document = new_document()
         parser = RstParser()
         parser.parse(rst, document)
-        SmartQuotes(document, startnode=None).apply()
-        for msg in document.traverse(nodes.system_message):
+        SphinxSmartQuotes(document, startnode=None).apply()
+        for msg in list(document.findall(nodes.system_message)):
             if msg['level'] == 1:
                 msg.replace_self([])
         return document
@@ -94,6 +94,7 @@ class ForgivingLaTeXTranslator(LaTeXTranslator, ForgivingTranslator):
 def verify_re_html(app, parse):
     def verify(rst, html_expected):
         document = parse(rst)
+        KeyboardTransform(document).apply()
         html_translator = ForgivingHTMLTranslator(document, app.builder)
         document.walkabout(html_translator)
         html_translated = ''.join(html_translator.fragment).strip()
@@ -108,7 +109,8 @@ def verify_re_latex(app, parse):
         app.builder = LaTeXBuilder(app)
         app.builder.set_environment(app.env)
         app.builder.init()
-        latex_translator = ForgivingLaTeXTranslator(document, app.builder)
+        theme = app.builder.themes.get('manual')
+        latex_translator = ForgivingLaTeXTranslator(document, app.builder, theme)
         latex_translator.first_document = -1  # don't write \begin{document}
         document.walkabout(latex_translator)
         latex_translated = ''.join(latex_translator.body).strip()
@@ -154,9 +156,10 @@ def get_verifier(verify, verify_re):
         'verify',
         ':pep:`8`',
         ('<p><span class="target" id="index-0"></span><a class="pep reference external" '
-         'href="http://www.python.org/dev/peps/pep-0008"><strong>PEP 8</strong></a></p>'),
-        ('\\index{Python Enhancement Proposals@\\spxentry{Python Enhancement Proposals}'
-         '!PEP 8@\\spxentry{PEP 8}}\\sphinxhref{http://www.python.org/dev/peps/pep-0008}'
+         'href="https://peps.python.org/pep-0008/"><strong>PEP 8</strong></a></p>'),
+        ('\\sphinxAtStartPar\n'
+         '\\index{Python Enhancement Proposals@\\spxentry{Python Enhancement Proposals}'
+         '!PEP 8@\\spxentry{PEP 8}}\\sphinxhref{https://peps.python.org/pep-0008/}'
          '{\\sphinxstylestrong{PEP 8}}')
     ),
     (
@@ -164,11 +167,12 @@ def get_verifier(verify, verify_re):
         'verify',
         ':pep:`8#id1`',
         ('<p><span class="target" id="index-0"></span><a class="pep reference external" '
-         'href="http://www.python.org/dev/peps/pep-0008#id1">'
+         'href="https://peps.python.org/pep-0008/#id1">'
          '<strong>PEP 8#id1</strong></a></p>'),
-        ('\\index{Python Enhancement Proposals@\\spxentry{Python Enhancement Proposals}'
+        ('\\sphinxAtStartPar\n'
+         '\\index{Python Enhancement Proposals@\\spxentry{Python Enhancement Proposals}'
          '!PEP 8\\#id1@\\spxentry{PEP 8\\#id1}}\\sphinxhref'
-         '{http://www.python.org/dev/peps/pep-0008\\#id1}'
+         '{https://peps.python.org/pep-0008/\\#id1}'
          '{\\sphinxstylestrong{PEP 8\\#id1}}')
     ),
     (
@@ -176,9 +180,10 @@ def get_verifier(verify, verify_re):
         'verify',
         ':rfc:`2324`',
         ('<p><span class="target" id="index-0"></span><a class="rfc reference external" '
-         'href="http://tools.ietf.org/html/rfc2324.html"><strong>RFC 2324</strong></a></p>'),
-        ('\\index{RFC@\\spxentry{RFC}!RFC 2324@\\spxentry{RFC 2324}}'
-         '\\sphinxhref{http://tools.ietf.org/html/rfc2324.html}'
+         'href="https://datatracker.ietf.org/doc/html/rfc2324.html"><strong>RFC 2324</strong></a></p>'),
+        ('\\sphinxAtStartPar\n'
+         '\\index{RFC@\\spxentry{RFC}!RFC 2324@\\spxentry{RFC 2324}}'
+         '\\sphinxhref{https://datatracker.ietf.org/doc/html/rfc2324.html}'
          '{\\sphinxstylestrong{RFC 2324}}')
     ),
     (
@@ -186,10 +191,11 @@ def get_verifier(verify, verify_re):
         'verify',
         ':rfc:`2324#id1`',
         ('<p><span class="target" id="index-0"></span><a class="rfc reference external" '
-         'href="http://tools.ietf.org/html/rfc2324.html#id1">'
+         'href="https://datatracker.ietf.org/doc/html/rfc2324.html#id1">'
          '<strong>RFC 2324#id1</strong></a></p>'),
-        ('\\index{RFC@\\spxentry{RFC}!RFC 2324\\#id1@\\spxentry{RFC 2324\\#id1}}'
-         '\\sphinxhref{http://tools.ietf.org/html/rfc2324.html\\#id1}'
+        ('\\sphinxAtStartPar\n'
+         '\\index{RFC@\\spxentry{RFC}!RFC 2324\\#id1@\\spxentry{RFC 2324\\#id1}}'
+         '\\sphinxhref{https://datatracker.ietf.org/doc/html/rfc2324.html\\#id1}'
          '{\\sphinxstylestrong{RFC 2324\\#id1}}')
     ),
     (
@@ -198,14 +204,14 @@ def get_verifier(verify, verify_re):
         '``code   sample``',
         ('<p><code class="(samp )?docutils literal notranslate"><span class="pre">'
          'code</span>&#160;&#160; <span class="pre">sample</span></code></p>'),
-        r'\\sphinxcode{\\sphinxupquote{code   sample}}',
+        r'\\sphinxAtStartPar\n\\sphinxcode{\\sphinxupquote{code   sample}}',
     ),
     (
         # interpolation of arrows in menuselection
         'verify',
         ':menuselection:`a --> b`',
         ('<p><span class="menuselection">a \N{TRIANGULAR BULLET} b</span></p>'),
-        '\\sphinxmenuselection{a \\(\\rightarrow\\) b}',
+        '\\sphinxAtStartPar\n\\sphinxmenuselection{a \\(\\rightarrow\\) b}',
     ),
     (
         # interpolation of ampersands in menuselection
@@ -213,7 +219,9 @@ def get_verifier(verify, verify_re):
         ':menuselection:`&Foo -&&- &Bar`',
         ('<p><span class="menuselection"><span class="accelerator">F</span>oo '
          '-&amp;- <span class="accelerator">B</span>ar</span></p>'),
-        r'\sphinxmenuselection{\sphinxaccelerator{F}oo \sphinxhyphen{}\&\sphinxhyphen{} \sphinxaccelerator{B}ar}',
+        ('\\sphinxAtStartPar\n'
+         r'\sphinxmenuselection{\sphinxaccelerator{F}oo \sphinxhyphen{}'
+         r'\&\sphinxhyphen{} \sphinxaccelerator{B}ar}'),
     ),
     (
         # interpolation of ampersands in guilabel
@@ -221,21 +229,77 @@ def get_verifier(verify, verify_re):
         ':guilabel:`&Foo -&&- &Bar`',
         ('<p><span class="guilabel"><span class="accelerator">F</span>oo '
          '-&amp;- <span class="accelerator">B</span>ar</span></p>'),
-        r'\sphinxguilabel{\sphinxaccelerator{F}oo \sphinxhyphen{}\&\sphinxhyphen{} \sphinxaccelerator{B}ar}',
+        ('\\sphinxAtStartPar\n'
+         r'\sphinxguilabel{\sphinxaccelerator{F}oo \sphinxhyphen{}\&\sphinxhyphen{} \sphinxaccelerator{B}ar}'),
     ),
     (
         # no ampersands in guilabel
         'verify',
         ':guilabel:`Foo`',
         '<p><span class="guilabel">Foo</span></p>',
-        r'\sphinxguilabel{Foo}',
+        '\\sphinxAtStartPar\n\\sphinxguilabel{Foo}',
     ),
     (
         # kbd role
         'verify',
         ':kbd:`space`',
         '<p><kbd class="kbd docutils literal notranslate">space</kbd></p>',
-        '\\sphinxkeyboard{\\sphinxupquote{space}}',
+        '\\sphinxAtStartPar\n\\sphinxkeyboard{\\sphinxupquote{space}}',
+    ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`Control+X`',
+        ('<p><kbd class="kbd compound docutils literal notranslate">'
+         '<kbd class="kbd docutils literal notranslate">Control</kbd>'
+         '+'
+         '<kbd class="kbd docutils literal notranslate">X</kbd>'
+         '</kbd></p>'),
+        '\\sphinxAtStartPar\n\\sphinxkeyboard{\\sphinxupquote{Control+X}}',
+    ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`Alt+^`',
+        ('<p><kbd class="kbd compound docutils literal notranslate">'
+         '<kbd class="kbd docutils literal notranslate">Alt</kbd>'
+         '+'
+         '<kbd class="kbd docutils literal notranslate">^</kbd>'
+         '</kbd></p>'),
+        ('\\sphinxAtStartPar\n'
+         '\\sphinxkeyboard{\\sphinxupquote{Alt+\\textasciicircum{}}}'),
+    ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`M-x  M-s`',
+        ('<p><kbd class="kbd compound docutils literal notranslate">'
+         '<kbd class="kbd docutils literal notranslate">M</kbd>'
+         '-'
+         '<kbd class="kbd docutils literal notranslate">x</kbd>'
+         '  '
+         '<kbd class="kbd docutils literal notranslate">M</kbd>'
+         '-'
+         '<kbd class="kbd docutils literal notranslate">s</kbd>'
+         '</kbd></p>'),
+        ('\\sphinxAtStartPar\n'
+         '\\sphinxkeyboard{\\sphinxupquote{M\\sphinxhyphen{}x  M\\sphinxhyphen{}s}}'),
+    ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`-`',
+        '<p><kbd class="kbd docutils literal notranslate">-</kbd></p>',
+        ('\\sphinxAtStartPar\n'
+         '\\sphinxkeyboard{\\sphinxupquote{\\sphinxhyphen{}}}'),
+    ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`Caps Lock`',
+        '<p><kbd class="kbd docutils literal notranslate">Caps Lock</kbd></p>',
+        ('\\sphinxAtStartPar\n'
+         '\\sphinxkeyboard{\\sphinxupquote{Caps Lock}}'),
     ),
     (
         # non-interpolation of dashes in option role
@@ -243,14 +307,15 @@ def get_verifier(verify, verify_re):
         ':option:`--with-option`',
         ('<p><code( class="xref std std-option docutils literal notranslate")?>'
          '<span class="pre">--with-option</span></code></p>$'),
-        r'\\sphinxcode{\\sphinxupquote{\\sphinxhyphen{}\\sphinxhyphen{}with\\sphinxhyphen{}option}}$',
+        (r'\\sphinxAtStartPar\n'
+         r'\\sphinxcode{\\sphinxupquote{\\sphinxhyphen{}\\sphinxhyphen{}with\\sphinxhyphen{}option}}$'),
     ),
     (
         # verify smarty-pants quotes
         'verify',
         '"John"',
         '<p>“John”</p>',
-        "“John”",
+        "\\sphinxAtStartPar\n“John”",
     ),
     (
         # ... but not in literal text
@@ -258,21 +323,21 @@ def get_verifier(verify, verify_re):
         '``"John"``',
         ('<p><code class="docutils literal notranslate"><span class="pre">'
          '&quot;John&quot;</span></code></p>'),
-        '\\sphinxcode{\\sphinxupquote{"John"}}',
+        '\\sphinxAtStartPar\n\\sphinxcode{\\sphinxupquote{"John"}}',
     ),
     (
         # verify classes for inline roles
         'verify',
         ':manpage:`mp(1)`',
         '<p><em class="manpage">mp(1)</em></p>',
-        '\\sphinxstyleliteralemphasis{\\sphinxupquote{mp(1)}}',
+        '\\sphinxAtStartPar\n\\sphinxstyleliteralemphasis{\\sphinxupquote{mp(1)}}',
     ),
     (
         # correct escaping in normal mode
         'verify',
         'Γ\\\\∞$',
         None,
-        'Γ\\textbackslash{}\\(\\infty\\)\\$',
+        '\\sphinxAtStartPar\nΓ\\textbackslash{}\\(\\infty\\)\\$',
     ),
     (
         # in verbatim code fragments
@@ -288,7 +353,7 @@ def get_verifier(verify, verify_re):
         'verify_re',
         '`test <https://www.google.com/~me/>`_',
         None,
-        r'\\sphinxhref{https://www.google.com/~me/}{test}.*',
+        r'\\sphinxAtStartPar\n\\sphinxhref{https://www.google.com/~me/}{test}.*',
     ),
     (
         # description list: simple
@@ -309,8 +374,12 @@ def get_verifier(verify, verify_re):
         # glossary (description list): multiple terms
         'verify',
         '.. glossary::\n\n   term1\n   term2\n       description',
-        ('<dl class="glossary docutils">\n<dt id="term-term1">term1</dt>'
-         '<dt id="term-term2">term2</dt><dd>description</dd>\n</dl>'),
+        ('<dl class="glossary docutils">\n'
+         '<dt id="term-term1">term1<a class="headerlink" href="#term-term1"'
+         ' title="Permalink to this term">¶</a></dt>'
+         '<dt id="term-term2">term2<a class="headerlink" href="#term-term2"'
+         ' title="Permalink to this term">¶</a></dt>'
+         '<dd>description</dd>\n</dl>'),
         None,
     ),
 ])
@@ -319,6 +388,21 @@ def test_inline(get_verifier, type, rst, html_expected, latex_expected):
     verifier(rst, html_expected, latex_expected)
 
 
+@pytest.mark.parametrize('type,rst,html_expected,latex_expected', [
+    (
+        'verify',
+        r'4 backslashes \\\\',
+        r'<p>4 backslashes \\</p>',
+        None,
+    ),
+])
+@pytest.mark.skipif(docutils.__version_info__ < (0, 16),
+                    reason='docutils-0.16 or above is required')
+def test_inline_docutils16(get_verifier, type, rst, html_expected, latex_expected):
+    verifier = get_verifier(type)
+    verifier(rst, html_expected, latex_expected)
+
+
 @pytest.mark.sphinx(confoverrides={'latex_engine': 'xelatex'})
 @pytest.mark.parametrize('type,rst,html_expected,latex_expected', [
     (
